@@ -27,15 +27,51 @@ import cfg, utils
 import webbrowser
 from osutils import *
 
-class RunExternalFailed(Exception):
-    def __init__(self, v):
-        self.v = v
-    def __str__(self):
-        return "\nRunning external program failed.\nPlease check your setup.\nThe command that failed was: '%s'" % (" ".join(self.v[1:]))
-
 def linux_play_wav_file(filename):
     run_external_program(cfg.get_string('sound/wav_player'), "", filename)
 
+def check_rcfile():
+    """See default.config for rcfileversion values, meanings and
+    a description of how to add config variables.
+    """
+    rcfileversion = 9
+    if cfg.get_int("app/rcfileversion") > rcfileversion:
+        cfg.drop_user_config()
+        return
+    if cfg.get_int("app/rcfileversion") <= 1:
+        if not "example-files" in cfg.get_string('config/lessoncollections'):
+            cfg.set_string('config/lessoncollections', 
+                "%s example-files" % cfg.get_string('config/lessoncollections'))
+    if cfg.get_int("app/rcfileversion") <= 5:
+        # This is more complicated that necessary to fix an old
+        # error.
+        if cfg.get_string("sound/commandline"):
+            cfg.del_key("sound/commandline")
+    if cfg.get_int("app/rcfileversion") <= 3:
+        cfg.set_list("config/lessoncollections",
+            cfg.get_string("config/lessoncollections").split())
+    if cfg.get_int("app/rcfileversion") <= 4:
+        cfg.del_key("config/web_browser")
+    if sys.platform == 'win32':
+        if cfg.get_string('sound/wav_player'):
+            cfg.del_key('sound/wav_player')
+    if cfg.get_int("app/rcfileversion") <= 5:
+        cfg.set_string("mainwin/history_back_ak", "<alt>Left")
+        cfg.set_string("mainwin/history_forward_ak", "<alt>Right")
+        cfg.set_string("mainwin/history_reload_ak", "<ctrl>r")
+    if cfg.get_int("app/rcfileversion") <= 6:
+        cfg.set_list("config/lessoncollections", ['solfege', 'user'])
+    if cfg.get_int("app/rcfileversion") <= 7:
+        cfg.set_int("rhythm/countin_perc", 80)
+    if cfg.get_int("app/rcfileversion") <= 8:
+        cfg.del_key("singinterval/highest_tone")
+        cfg.del_key("singinterval/lowest_tone")
+        cfg.del_key("melodicinterval/highest_tone")
+        cfg.del_key("melodicinterval/lowest_tone")
+        cfg.del_key("harmonicinterval/highest_tone")
+        cfg.del_key("harmonicinterval/lowest_tone")
+    cfg.set_int("app/rcfileversion", rcfileversion)
+
 class SolfegeApp(cfg.ConfigUtils):
     def __init__(self, options, ui, lessonfile_manager):
         """
@@ -46,53 +82,11 @@ class SolfegeApp(cfg.ConfigUtils):
         self.m_test_mode = False
         self.m_options = options
         self.m_ui = ui
-        self.check_rcfile()
         self.m_teachers = {}
         self.m_running_exercise = None
         self.lessonfile_manager = lessonfile_manager
         self.m_sound_init_exception = None
         self.setup_sound()
-    def check_rcfile(self):
-        """See default.config for rcfileversion values, meanings and
-        a description of how to add config variables.
-        """
-        rcfileversion = 9
-        if cfg.get_int("app/rcfileversion") > rcfileversion:
-            cfg.drop_user_config()
-            return
-        if cfg.get_int("app/rcfileversion") <= 1:
-            if not "example-files" in cfg.get_string('config/lessoncollections'):
-                cfg.set_string('config/lessoncollections', 
-                    "%s example-files" % cfg.get_string('config/lessoncollections'))
-        if cfg.get_int("app/rcfileversion") <= 5:
-            # This is more complicated that necessary to fix an old
-            # error.
-            if cfg.get_string("sound/commandline"):
-                cfg.del_key("sound/commandline")
-        if cfg.get_int("app/rcfileversion") <= 3:
-            cfg.set_list("config/lessoncollections",
-                cfg.get_string("config/lessoncollections").split())
-        if cfg.get_int("app/rcfileversion") <= 4:
-            cfg.del_key("config/web_browser")
-        if sys.platform == 'win32':
-            if cfg.get_string('sound/wav_player'):
-                cfg.del_key('sound/wav_player')
-        if cfg.get_int("app/rcfileversion") <= 5:
-            cfg.set_string("mainwin/history_back_ak", "<alt>Left")
-            cfg.set_string("mainwin/history_forward_ak", "<alt>Right")
-            cfg.set_string("mainwin/history_reload_ak", "<ctrl>r")
-        if cfg.get_int("app/rcfileversion") <= 6:
-            cfg.set_list("config/lessoncollections", ['solfege', 'user'])
-        if cfg.get_int("app/rcfileversion") <= 7:
-            cfg.set_int("rhythm/countin_perc", 80)
-        if cfg.get_int("app/rcfileversion") <= 8:
-            cfg.del_key("singinterval/highest_tone")
-            cfg.del_key("singinterval/lowest_tone")
-            cfg.del_key("melodicinterval/highest_tone")
-            cfg.del_key("melodicinterval/lowest_tone")
-            cfg.del_key("harmonicinterval/highest_tone")
-            cfg.del_key("harmonicinterval/lowest_tone")
-        cfg.set_int("app/rcfileversion", rcfileversion)
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
