@@ -19,7 +19,8 @@ import gtk, gnome.ui
 import gobject
 import random
 
-import gu, widgets, utils
+import gu, utils
+from multipleintervalconfigwidget import IntervalCheckBox
 import mpd, mpd.musicdisplayer, soundcard
 import abstract, const
 import lessonfile
@@ -155,11 +156,6 @@ class Gui(abstract.Gui):
         ##############
         # config_box #
         ##############
-        self.g_instrument_configurator \
-                 = widgets.InstrumentConfigurator(self.m_exname, 2,
-                    _("_Change midi instrument for this exercise"))
-        self.config_box.pack_start(self.g_instrument_configurator, False)
-        # ------------------------------------------
         self._add_auto_new_question_gui(self.config_box)
         # ----------------------------------------------
 
@@ -184,10 +180,10 @@ class Gui(abstract.Gui):
         hbox.pack_start(gtk.Label(_("First interval:")), False,
                         padding=gnome.ui.PAD_SMALL)
         self.g_rdbs = [pack_rdbs(hbox, self.update_first)]
-        self.g_first_interval_up = widgets.IntervalCheckBox(self.m_exname,
+        self.g_first_interval_up = IntervalCheckBox(self.m_exname,
                                                          'first_interval_up')
         self.g_intervalconfig_box.pack_start(self.g_first_interval_up, False)
-        self.g_first_interval_down = widgets.IntervalCheckBox(self.m_exname,
+        self.g_first_interval_down = IntervalCheckBox(self.m_exname,
                              'first_interval_down')
         self.g_intervalconfig_box.pack_start(self.g_first_interval_down, False)
         #----------
@@ -195,10 +191,10 @@ class Gui(abstract.Gui):
         hbox.pack_start(gtk.Label(_("Last interval:")), False,
                         padding=gnome.ui.PAD_SMALL)
         self.g_rdbs.append (pack_rdbs(hbox, self.update_last))
-        self.g_last_interval_up = widgets.IntervalCheckBox(self.m_exname,
+        self.g_last_interval_up = IntervalCheckBox(self.m_exname,
                      'last_interval_up')
         self.g_intervalconfig_box.pack_start(self.g_last_interval_up, False)
-        self.g_last_interval_down = widgets.IntervalCheckBox(self.m_exname,
+        self.g_last_interval_down = IntervalCheckBox(self.m_exname,
                         'last_interval_down')
         self.g_intervalconfig_box.pack_start(self.g_last_interval_down, False)
         #------------
@@ -320,9 +316,16 @@ class Gui(abstract.Gui):
         elif q == Teacher.ERR_PICKY:
             self.g_flashbar.flash(_("You have to solve this question first."))
         else:
-            assert q == Teacher.ERR_CONFIGURE
+            assert q == Teacher.ERRp_CONFIGURE
             self.g_flashbar.flash(_("You have to configure the exercise properly"))
     def on_start_practise(self):
+        for n in ('first_interval_up', 'first_interval_down',
+                  'last_interval_up', 'last_interval_down'):
+            if n in self.m_t.m_P.header:
+                self.set_list(n, self.m_t.m_P.header[n])
+        for n in ('first_interval_type', 'last_interval_type'):
+            if n in self.m_t.m_P.header:
+                self.set_string(n, self.m_t.m_P.header[n])
         gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
             self.g_flashbar.flash(_("Click 'New' to begin.")))
         self.g_new.grab_focus()
