@@ -15,14 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-try:
-    app_running == True
-except:
-    import sys
-    sys.path.append("src")
-    sys.path.append(".")
-    import i18n
-    i18n.setup_srcdir()
+if __name__ == '__main__':
+    import testsetup
 
 import xrandom
 import uuid
@@ -62,7 +56,7 @@ import utils
 from soundcard.rat import Rat
 import soundcard
 import dataparser
-import app
+import osutils
 
 # The keys in the dict say how many steps up or down in the
 # circle of fifths we go if we transpose.
@@ -103,6 +97,8 @@ predef = {'dictation': 'dictation',
           'melodicinterval': 'melodicinterval',
           'harmonicinterval': 'harmonicinterval',
           'idtone': 'idtone',
+          'twelvetone': 'twelvetone',
+          'identifybpm': 'identifybpm',
           #
           'progression': 'progression',
           'normal': 'normal',
@@ -135,14 +131,13 @@ lessonfile_functions = {
         'wavfile': lambda m: Music(m, 'wavfile'),
 }
 
-class _Header:
+class _Header(dict):
     def __init__(self, headerdict):
-        self.m_headerdict = headerdict
+        dict.__init__(self, headerdict)
         for key, value in (
                            ('version', ''),
                            ('title', ''),
                            ('description', ''),
-                           #('content', ''), This MUST be set in the lessonfile
                            ('musicformat', 'normal'),
                            ('random_transpose', True),
                            ('labelformat', 'normal'),
@@ -154,20 +149,22 @@ class _Header:
                            ('have_music_displayer', False),
                            ('enable_right_click', True),
                            ('disable_unused_intervals', True),
-                           # use only by the rhythm_exercise
-                           ('num_beats', 4.0),
-                           ('bpm', 60),
-                           ('countin_perc', 75),
-                           ('rhythm_perc', 37),
-                           ('count_in', 2.0),
-                           ('not_start_with_rest', True),
                            ):
-            if key not in self.m_headerdict:
-                self.m_headerdict[key] = value
+            if key not in self:
+                self[key] = value
     def __getattr__(self, name):
-        if name in self.m_headerdict:
-            return dataparser.get_translated_string(self.m_headerdict, name)
-        print "warning: lessonfile.Header.%s not set. Returning empty string" % name
+        """
+        This function let us write
+
+          header.variable_name
+
+        as a shortcut or:
+
+            header['variable_name']
+        """
+        if name in self:
+            return dataparser.get_translated_string(self, name)
+        # FIXME print "warning: lessonfile.Header.%s not set. Returning empty string" % name
         return ""
         #raise NameError(name)
 
@@ -518,7 +515,12 @@ class LessonfileCommon(object):
         self.m_test_idx += 1
         self._idx = self.m_test_questions[self.m_test_idx]
         if self.header.random_transpose:
-            self.set_random_transpose()
+            old = self.m_transpose
+            # try really hard not to get the same tonika:
+            for x in range(100):
+                self.set_random_transpose()
+                if old != self.m_transpose:
+                    break
     def is_test_complete(self):
         """
         Return True if the test is compleded.
@@ -531,7 +533,7 @@ class LessonfileCommon(object):
         if not question:
             question = self.get_question()
         if question[varname].m_musictype == 'cmdline':
-            app.run_external_program(str(question[varname].m_musicdata), cfg.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')), "")
+            osutils.run_external_program(str(question[varname].m_musicdata), cfg.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')), "")
         elif question[varname].m_musictype == 'wavfile':
             soundcard.play_wav_file(os.path.join(
                 self.m_t.get_string('lessoncollections/%s' % self.m_t.get_string('lessoncollection')),
@@ -847,12 +849,9 @@ class LessonFileManager:
     def __init__(self):
         self.parse()
     def parse(self):
-        self.m_db = {}
         self.m_uiddb = {}
-        self.m_invalid_files = []
         vim_tmpfile_re = re.compile("\..*\.swp")
         for collection in cfg.get_list("config/lessoncollections"):
-            self.m_db[collection] = {}
             if not os.path.isdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
                 continue
             # filter out backup files
@@ -862,53 +861,30 @@ class LessonFileManager:
                 if filename in ('CVS', 'Makefile', 'Makefile.in') \
                   or not os.path.isfile(self.get_complete_filename(collection, filename)):
                     continue
+                self.parse_into_uiddb(collection, filename)
+    def parse_into_uiddb(self, collection, filename):
+                fn = self.get_complete_filename(collection, filename)
                 try:
-                    f = LessonIdParser(self.get_complete_filename(collection, filename))
+                    f = LessonIdParser(fn)
                     if not f.has_lesson_id():
                         f.add_lesson_id()
                 except IOError, e:
-                    print >> sys.stderr, "add_lesson_id('%s'):" % self.get_complete_filename(collection, filename), e
-                    continue
+                    print >> sys.stderr, "add_lesson_id('%s'):" % fn, e
+                    return
                 try:
                     p = dataparser.Dataparser(predef, lessonfile_functions)
-                    p.parse_file(self.get_complete_filename(collection, filename))
+                    p.parse_file(fn)
                 except dataparser.DataparserException, e:
-                    self.m_invalid_files.append((collection, filename))
-                    print >> sys.stderr, "LessonFileManager.parse: Discarding lessonfile '%s/%s' because the file contain errors:" % (collection, filename)
+                    print >> sys.stderr, "LessonFileManager.parse: Discarding lessonfile '%s' because the file contain errors:" % fn
                     print >> sys.stderr, str(e)
-                    continue
-                if 'title' in p.header:
-                    title = p.header['title']
-                else:
-                    title = ""
-                if 'url' in p.header:
-                    url = p.header['url']
-                    #p.header['content'] = utils.Url(p.header['url']).exercisemodule
-                else:
-                    url = None
-                if 'content' not in p.header:
-                    if 'module' not in p.header:
-                        print " Hopper over", filename
-                        continue
-                    p.header['content'] = p.header['module']
-                if isinstance(p.header['content'], basestring):
-                    p.header['content'] = [p.header['content']]
+                    return
                 if not 'title' in p.header:
                     p.header['title'] = '@@@NOT TITLE'
-                for u in p.header['content']:
-                    if u not in self.m_db[collection]:
-                        self.m_db[collection][u] = []
-                    self.m_db[collection][u].append({'filename': filename,
-                                                     'title': title,
-                                                     'lesson_id': p.header['lesson_id']}
-                                                )
                 n = {
-                   'filename': self.get_complete_filename(collection, filename),
-                   'lessoncollection': collection,#FIXME rename lessoncollection
+                   'filename': fn,
+                   'lessoncollection': collection,
                    'lessonfile': filename,
-                   'url': url,
-                   'lesson_id': p.header['lesson_id'],
-                   'mtime': os.stat(self.get_complete_filename(collection, filename))[stat.ST_MTIME],
+                   'mtime': os.stat(fn)[stat.ST_MTIME],
                    'header': p.header,
                 }
                 if not p.header['lesson_id'] in self.m_uiddb:
@@ -918,50 +894,52 @@ class LessonFileManager:
                         self.m_uiddb[p.header['lesson_id']] = \
                             [self.m_uiddb[p.header['lesson_id']]]
                     self.m_uiddb[p.header['lesson_id']].append(n)
-        for collection in self.m_db.itervalues():
-            for content in collection:
-                collection[content].sort()
+    def create_lessonfile_index(self):
         self.m_htmldoc = """<html></head><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"></head><body><p>%s</p>""" % _("This page lists all the lesson files that Solfege can find. You can click on the links to start practising.")
-        if self.m_invalid_files:
-            self.m_htmldoc += "<h1>%s</h1><ul>" % _("Files we failed to parse")
-            for collection, filename in self.m_invalid_files:
-                self.m_htmldoc += "<li>%s</li>" % (_("%(filename)s in collection %(collection)s") % {'collection': collection, 'filename': filename})
-            self.m_htmldoc += "</ul>"
-        for collection in self.m_db:
-            self.m_htmldoc = self.m_htmldoc + "<h1>%s: %s</h1>" % (_("Lesson collection"), collection)
-            for use in self.m_db[collection]:
-                self.m_htmldoc = self.m_htmldoc + "<h2>%s</h2><p>" % use
-                for d in self.m_db[collection][use]:
-                    fn = d['filename']
-                    self.m_htmldoc = self.m_htmldoc + \
-                      "<br><a href=\"solfege:practise/%(collection)s/%(fn)s\">%(fn)s</a>: %(title)s\n" % {'use':use, 'collection': collection,
-                        'fn': fn, 'title': d['title']}
-                self.m_htmldoc += "</p>"
-        self.m_htmldoc = self.m_htmldoc + "</body></html>"
+        d = {}
+        for lesson_id, n in self.m_uiddb.items():
+            if not 'module' in n['header']:
+                print "warning: %s is missing a module declaration" %  self.get(lesson_id, 'filename')
+                continue
+            if n['header']['module'] not in d:
+                d[n['header']['module']] = []
+            d[n['header']['module']].append(lesson_id)
+        for module in d.keys():
+            self.m_htmldoc = "%s<h2>%s</h2>" % (self.m_htmldoc, module)
+            self.m_htmldoc += "<ul>"
+            for n in d[module]:
+                if self.get(n, 'lessoncollection') != 'solfege':
+                    collection_info = "[%s]" % self.get(n, 'lessoncollection')
+                else:
+                    collection_info = ""
+                self.m_htmldoc += "<li><a href='solfege:practise/%s'>%s%s</a>: %s</li>" % (
+                    n,
+                    collection_info,
+                    self.get(n, 'lessonfile'),
+                    self.get(n, 'title'))
     def get(self, lesson_id, fieldname):
-        if fieldname in ('title', 'test', 'content', 'module'):
+        if fieldname in ('title', 'test', 'module'):
             if fieldname in self.m_uiddb[lesson_id]['header']:
                 return self.m_uiddb[lesson_id]['header'][fieldname]
-            elif fieldname == 'module':
-                print "warning: update %s to use the new 'module' variable" % self.get(lesson_id, 'filename')
-                s = self.m_uiddb[lesson_id]['header']['content'][0]
-                s = s.replace("-", "")
-                return s
         if fieldname in self.m_uiddb[lesson_id]:
             return self.m_uiddb[lesson_id][fieldname]
     def is_test_passed(self, lesson_id):
         return os.path.exists(os.path.join(gethomedir.get_home_dir(),
             '.solfege', 'testresults', lesson_id, 'passed'))
-    def ignore_files_with_lesson_id(self, lesson_id):
+    def ignore_duplicates_with_lesson_id(self, lesson_id):
         """
-        Delete lesson files from .m_db if they have the lesson_id lesson_id.
+        Delete the duplicates with lesson_id, keep the oldest file.
         """
-        for collection in self.m_db:
-            for use in self.m_db[collection]:
-                for d in self.m_db[collection][use]:
-                    if d['lesson_id'] == lesson_id:
-                        d['ignore'] = True
-                self.m_db[collection][use] = [d for d in self.m_db[collection][use] if 'ignore' not in d]
+        def sort_func(a, b):
+            return cmp(a['mtime'], b['mtime'])
+        self.m_uiddb[lesson_id].sort(sort_func)
+        self.m_uiddb[lesson_id] = self.m_uiddb[lesson_id][0]
+    def delete_not_fn(self, lesson_id, fn):
+        """
+        Assumes the lesson_id has duplicate entries.
+        Delete the entries that is not the file fn.
+        """
+        self.m_uiddb[lesson_id] = [d for d in self.m_uiddb[lesson_id] if d['filename'] == fn][0]
     def iterate_duplicated_lesson_id(self):
         """
         Return the a lesson_id if there exist lesson_ids that
@@ -969,7 +947,7 @@ class LessonFileManager:
         """
         for v in self.m_uiddb.values():
             if isinstance(v, list):
-                yield v[0]['lesson_id']
+                yield v[0]['header']['lesson_id']
     def iterate_lesson_ids(self):
         for k in self.m_uiddb:
             yield k
@@ -978,19 +956,7 @@ class LessonFileManager:
         FIXME: the function name is not very good.
         Return data used to fix things when we have a lesson_id crash.
         """
-        return [{'filename': d['filename'], 'timestr': time.strftime(locale.nl_langinfo(locale.D_T_FMT), time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
-    def get_href_from_uuid(self, lesson_id):
-        if 'url' in self.m_uiddb[lesson_id]['header']:
-            return self.m_uiddb[lesson_id]['header']['url']
-        return "solfege:practise/%s/%s/%s" % (
-            self.m_uiddb[lesson_id]['header']['content'][0],
-            self.m_uiddb[lesson_id]['collection'],
-            self.m_uiddb[lesson_id]['lessonfile'])
-    def get_collections(self):
-        """
-        Return a list of available collections. 
-        """
-        return self.m_db.keys()
+        return [{'filename': d['filename'], 'lessoncollection': d['lessoncollection'], 'lessonfile': d['lessonfile'], 'timestr': time.strftime(locale.nl_langinfo(locale.D_T_FMT), time.localtime(d['mtime'])), 'mtime': d['mtime']} for d in self.m_uiddb[lesson_id]]
     def get_complete_filename(self, catalog, fn):
         """
         Return the absolute path to a file in a given collection
@@ -999,19 +965,7 @@ class LessonFileManager:
             catalog = 'solfege'
         return os.path.expanduser(os.path.join(
          cfg.get_string('lessoncollections/%s' % catalog), fn))
-    def get_filenames(self, collection, usetype):
-        if usetype in self.m_db[collection]:
-            v = []
-            for d in self.m_db[collection][usetype]:
-                v.append(d['filename'])
-            return v
-        else:
-            return []
-    def get_info(self, collection, usetype):
-        if usetype in self.m_db[collection]:
-            return self.m_db[collection][usetype]
-        else:
-            return []
+
 
 class LessonIdParser(object):
     """
@@ -1041,7 +995,8 @@ class LessonIdParser(object):
             ofile = file(self.m_filename, 'w')
             m = re.search("lesson_id\s*=\s*\".+?\"", self.m_file_content)
             ofile.write(self.m_file_content[:m.start()])
-            ofile.write("lesson_id = \"%s\"" % uuid.generate())
+            self.m_new_id = uuid.generate()
+            ofile.write("lesson_id = \"%s\"" % self.m_new_id)
             ofile.write(self.m_file_content[m.end():])
             ofile.close()
         else:
@@ -1056,7 +1011,8 @@ class LessonIdParser(object):
         for i in range(len(v)):
             if not v[i].startswith('#'):
                 break
-        v.insert(i+1, "header { \n    lesson_id=\"%s\"\n}" % uuid.generate())
+        self.m_new_id = uuid.generate()
+        v.insert(i+1, "header { \n    lesson_id=\"%s\"\n}" % self.m_new_id)
         self.m_file_content = "\n".join(v)
         ofile = file(self.m_filename, 'w')
         ofile.write(self.m_file_content)
@@ -1075,7 +1031,8 @@ class LessonIdParser(object):
             if m:
                 ofile.write(self.m_file_content[:m.start()])
                 ofile.write("\nheader {")
-                ofile.write("\n    lesson_id = \"%s\"" % uuid.generate())
+                self.m_new_id = uuid.generate()
+                ofile.write("\n    lesson_id = \"%s\"" % self.m_new_id)
                 ofile.write(self.m_file_content[m.end():])
             ofile.close()
 
@@ -1090,7 +1047,7 @@ if __name__ == '__main__':
         #p.parse_file(sys.argv[1])
         #p = LessonFileManager()
         p = LessonfileCommon(None)
-        p.parse_file(sys.argv[1])
+        p.parse_file('lesson-files/chord-min-major')
         print p.m_globals
         for q in p.m_questions:
             print q
