@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-copyright = "Copyright © 1999-2005 Tom Cato Amundsen <tca@gnu.org>"
+copyright = "Copyright  1999-2005 Tom Cato Amundsen <tca@gnu.org>"
 
 import sys
 import os
@@ -557,7 +557,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
             self.g_about_window = gnome.ui.About("GNU Solfege",
                   configureoutput.VERSION_STRING,
-                  copyright,
+                  copyright.decode("iso-8859-1"),
                   _("GPL'ed ear training.") + "\n" + "http://www.solfege.org",
                   ["Tom Cato Amundsen",
                   'Oliver Vollmer %s' % _("(toolbar icons)"),
@@ -769,6 +769,10 @@ prog.parse_args()
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
 
+# check_rcfile has to be called before and
+# functions that use the cfg module.
+app.check_rcfile()
+
 cfg.set_bool('config/no_random', bool(options.no_random))
 
 if splash_win:
