@@ -16,7 +16,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import gtk, gnome, gobject
-import gu, widgets, inputwidgets
+import gu, inputwidgets
+from multipleintervalconfigwidget import MultipleIntervalConfigWidget
 import abstract, const
 import statistics, statisticsviewer
 import soundcard, utils, mpd
@@ -29,6 +30,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
         self.lessonfileclass = lessonfile.IdByNameLessonfile
         self.m_statistics = statistics.IntervalStatistics(self)
         self.m_timeout_handle = None
+        self.m_custom_mode = False
     def enter_test_mode(self):
         self.m_statistics.enter_test_mode()
         count, t = lessonfile.parse_test_def(self.m_P.header.test)
@@ -51,7 +53,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
             q = map(abs, self.m_question)
         if q == answer:
             if self.q_status == const.QSTATUS_NEW \
-                and not self.get_bool('gui/expert_mode') \
+                and not self.m_custom_mode \
                 and self.get_int('number_of_intervals=1') == 1:
                     self.m_statistics.add_correct(self.m_question[0])
             self.maybe_auto_new_question()
@@ -59,7 +61,7 @@ class Teacher(abstract.MelodicIntervalTeacher):
             return 1
         else:
             if self.q_status == const.QSTATUS_NEW \
-                and not self.get_bool('gui/expert_mode') \
+                and not self.m_custom_mode \
                 and self.get_int('number_of_intervals=1') == 1:
                     self.m_statistics.add_wrong(self.m_question[0], answer[0])
             if self.m_app.m_test_mode:
@@ -76,7 +78,7 @@ class Gui(abstract.IntervalGui):
         ##############
         # config_box #
         ##############
-        self.g_mici = widgets.MultipleIntervalConfigWidget(self.m_exname)
+        self.g_mici = MultipleIntervalConfigWidget(self.m_exname)
         self.config_box.pack_start(self.g_mici, False, False)
         self.config_box.pack_start(gtk.HBox(), False,
                                    padding=gnome.ui.PAD_SMALL)
@@ -109,7 +111,7 @@ class Gui(abstract.IntervalGui):
             # the user configures the exercise manually. We only have to run
             # end|start_exercise here in expert mode because it is called
             # automatically when we change lesson file.
-            if self.get_bool('gui/expert_mode'):
+            if self.m_t.m_custom_mode:
                 self.on_end_practise()
                 #self.on_start_practise()
         for i in range(self.get_int('maximum_number_of_intervals')):
@@ -230,13 +232,22 @@ class Gui(abstract.IntervalGui):
         elif ret == Teacher.ERR_PICKY:
             self.g_flashbar.flash(_("You have to solve this question first."))
     def on_start_practise(self):
-        for i in range(self.get_int('maximum_number_of_intervals')):
-            if self.m_t.m_P.header.__getattr__('ask_for_intervals_%i' % i):
-                self.set_list('ask_for_intervals_%i' % i,
-                  self.m_t.m_P.header.__getattr__('ask_for_intervals_%i' % i))
-        self.set_int('number_of_intervals', self.m_t.m_P.header.number_of_intervals)
-        self.set_bool('disable_unused_intervals', self.m_t.m_P.header.disable_unused_intervals)
-        if self.get_bool('gui/expert_mode'):
+        # If ask_for_intervals_0 is not set, then we run in custom_mode
+        # where the user configure the exercise on her own.
+        if 'ask_for_intervals_0' in self.m_t.m_P.header:
+            self.set_int('number_of_intervals', self.m_t.m_P.header.get('number_of_intervals', 1))
+            self.set_bool('disable_unused_intervals', self.m_t.m_P.header.disable_unused_intervals)
+            for i in range(self.get_int('maximum_number_of_intervals')):
+                if 'ask_for_intervals_%i' % i in self.m_t.m_P.header:
+                    self.set_list('ask_for_intervals_%i' % i,
+                      self.m_t.m_P.header['ask_for_intervals_%i' % i])
+        if self.m_t.m_P.header.lesson_heading:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(_("Identify the interval"))
+        self.m_t.m_custom_mode = bool(
+            not self.m_t.m_P.header.ask_for_intervals_0)
+        if self.m_t.m_custom_mode:
             self.g_mici.show()
         else:
             self.m_t.m_statistics.reset_session()
