@@ -15,28 +15,28 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'sing-interval',
-    'stock-icon': 'solfege-sing-interval',
-    'menupath': '/Menubar/FileMenu/IntervalsMenu',
-    'name': _("_Sing interval"),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gtk, gnome
-import gu, widgets
+import gu
+from multipleintervalconfigwidget import MultipleIntervalConfigWidget
 import soundcard, mpd, mpd.musicdisplayer
 import abstract, const
 import statistics, statisticsviewer
 import utils
+import lessonfile
+import random
 
 class Teacher(abstract.MelodicIntervalTeacher):
-    exercise_data = __exercise_data__
-    def __init__(self, exname, app, config):
-        abstract.MelodicIntervalTeacher.__init__(self, exname, app, config)
+    def __init__(self, exname, app):
+        abstract.MelodicIntervalTeacher.__init__(self, exname, app)
+        self.lessonfileclass = lessonfile.IdByNameLessonfile
         self._ignore_picky = 1
         self.m_statistics = statistics.IntervalStatistics(self)
+    def enter_test_mode(self):
+        self.m_statistics.enter_test_mode()
+        count, t = lessonfile.parse_test_def(self.m_P.header.test)
+        self.m_test_questions = self.get_list('ask_for_intervals_0') * count
+        random.shuffle(self.m_test_questions)
+        self.m_test_idx = -1
     def get_question(self):
         s = self.m_tonika.get_octave_notename()
         last = self.m_tonika
@@ -47,11 +47,13 @@ class Teacher(abstract.MelodicIntervalTeacher):
             s = s + " " + last.get_octave_notename()
         return self.m_tonika, r"\staff{ \clef %s %s}" % (mpd.select_clef(s), s)
     def guessed_correct(self):
-        if self.get_int('number_of_intervals=1') == 1:
+        if self.get_int('number_of_intervals=1') == 1 \
+                and not self.m_custom_mode:
             self.m_statistics.add_correct(self.m_question[0])
         self.q_status = const.QSTATUS_SOLVED
     def guessed_wrong(self):
-        if self.get_int('number_of_intervals=1') == 1:
+        if self.get_int('number_of_intervals=1') == 1 \
+                and not self.m_custom_mode:
             self.m_statistics.add_wrong(self.m_question[0], None)
         # we set q_status to WRONG, just to trick the MelodicIntervalTeacher
         # .new_interval to do the work for us.
@@ -109,7 +111,7 @@ class Gui(abstract.Gui):
         ##############
         # config_box #
         ##############
-        self.g_mici = widgets.MultipleIntervalConfigWidget(self.m_exname)
+        self.g_mici = MultipleIntervalConfigWidget(self.m_exname)
         self.config_box.pack_start(self.g_mici, False)
         self.config_box.show_all()
         ###############
@@ -138,7 +140,26 @@ class Gui(abstract.Gui):
             self.g_score_displayer.clear()
         return r
     def on_start_practise(self):
-        self.m_t.m_statistics.reset_session()
+        # If ask_for_intervals_0 is not set, then we run in custom_mode
+        # where the user configure the exercise on her own.
+        if self.m_t.m_P.header.ask_for_intervals_0:
+            for i in range(self.get_int('maximum_number_of_intervals')):
+                if 'ask_for_intervals_%i' % i in self.m_t.m_P.header:
+                    self.set_list('ask_for_intervals_%i' % i,
+                      self.m_t.m_P.header['ask_for_intervals_%i' % i])
+            self.set_int('number_of_intervals', self.m_t.m_P.header.number_of_intervals)
+        self.m_t.m_custom_mode = bool(
+                      not self.m_t.m_P.header.ask_for_intervals_0)
+        if self.m_t.m_P.header.lesson_heading:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(_("Sing the interval"))
+        if self.m_t.m_custom_mode:
+            self.g_mici.show()
+        else:
+            self.g_mici.hide()
+            self.m_t.m_statistics.reset_session()
+        self.handle_config_box_visibility()
         self.g_new_interval.grab_focus()
     def on_end_practise(self):
         self.g_new_interval.show()
@@ -157,9 +178,24 @@ class Gui(abstract.Gui):
                  # teacher when we get a new question automatically.
         if self.m_t.q_status == const.QSTATUS_NEW:
             self.m_t.guessed_correct()
+            if self.m_t.m_app.m_test_mode and self.m_t.is_test_complete():
+                self.do_test_complete()
+                return
         r = self._new_question()
         if r == Teacher.OK:
             self.g_new_interval.hide()
             self.g_new_interval_correct.show()
             self.g_new_interval_wrong.show()
-
+    def enter_test_mode(self):
+        self.m_saved_q_auto = self.get_bool('new_question_automatically')
+        self.m_saved_s_new = self.get_float('seconds_before_new_question')
+        self.set_bool('new_question_automatically', True)
+        self.set_float('seconds_before_new_question', 0.5)
+        self.m_t.enter_test_mode()
+        self.g_new_interval.set_label(_("_Start test"))
+    def exit_test_mode(self):
+        self.set_bool('new_question_automatically', self.m_saved_q_auto)
+        self.set_float('seconds_before_new_question', self.m_saved_s_new)
+        self.m_t.exit_test_mode()
+        self.g_new_interval.show()
+        self.g_new_interval.set_label(_("_New interval"))
